package org.eclipse.mylar.zest.layouts.algorithms;

import java.awt.geom.Point2D;
import java.util.Iterator;

import org.eclipse.mylar.zest.layouts.dataStructures.InternalNode;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalRelationship;

import edu.uci.ics.jung.graph.Edge;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.Vertex;
import edu.uci.ics.jung.graph.impl.DirectedSparseEdge;
import edu.uci.ics.jung.graph.impl.SparseGraph;
import edu.uci.ics.jung.graph.impl.SparseVertex;
import edu.uci.ics.jung.utils.UserDataContainer;
import edu.uci.ics.jung.visualization.Layout;

/**
 * Parent class for all Jung layouts 
 * @author Ian Bull
 */
public abstract class AbstractJungLayout extends AbstractLayoutAlgorithm {

	protected Graph graph = null;
	protected Layout layout = null;

	public AbstractJungLayout(int styles) {
		super(styles);
		// TODO Auto-generated constructor stub
	}

	protected int getCurrentLayoutStep() {
		// TODO Auto-generated method stub
		return 0;
	}

	protected int getTotalNumberOfLayoutSteps() {
		// TODO Auto-generated method stub
		return 0;
	}

	/**
	 * Is this a valid combination of asynch and continuous params for this type of layout.
	 * Note - we may want to set this in the subclasses.
	 */
	protected boolean isValidConfiguration(boolean asynchronous,
			boolean continuous) {
		if (asynchronous && continuous)
			return false;
		else if (asynchronous && !continuous)
			return true;
		else if (!asynchronous && continuous)
			return false;
		else if (!asynchronous && !continuous)
			return true;
		return false;
	}

	/**
	 * Post-processing of the layout. 
	 */
	protected void postLayoutAlgorithm(InternalNode[] entitiesToLayout,
			InternalRelationship[] relationshipsToConsider) {

		Iterator iterator = layout.getVertexIterator();
		while ( iterator.hasNext() ) {
			Vertex v = (Vertex) iterator.next();
			Point2D p = layout.getLocation(v);
			InternalNode internalNode = (InternalNode)v.getUserDatum("internalNode".intern());
			internalNode.setLocation(p.getX(), p.getY());
		}
	}


	/**
	 * Set up the graph for layout.
	 */
	protected void preLayoutAlgorithm(InternalNode[] entitiesToLayout,
			InternalRelationship[] relationshipsToConsider, double x, double y,
			double width, double height) {
		graph = new SparseGraph();

		for ( int i = 0; i < entitiesToLayout.length; i++ ) {
			Vertex v = new SparseVertex();
			InternalNode internalNode = entitiesToLayout[i];
			v.setUserDatum("internalNode".intern(), internalNode, new UserDataContainer.CopyAction.Clone() );
			internalNode.setAttributeInLayout("vertex".intern(),v);
			graph.addVertex(v);
		}
		
		for ( int i = 0; i < relationshipsToConsider.length; i++ ) {
			InternalRelationship internalRelationship = relationshipsToConsider[i];
			InternalNode source = internalRelationship.getSource();
			InternalNode destination = internalRelationship.getDestination();
			Vertex sourceV = (Vertex) source.getAttributeInLayout("vertex".intern());
			Vertex destinationV = (Vertex) destination.getAttributeInLayout("vertex".intern());
			Edge e = new DirectedSparseEdge(sourceV, destinationV);
			graph.addEdge(e);	
		}

	}

	/**
	 * Set the physical layout area
	 */
	public void setLayoutArea(double x, double y, double width, double height) {
		// Do nothing - bounds are taken care of by the layout processing in each subclass 
	}

}
